==> jmb.filters-0.3.tar.gz: jmb.filters-0.3/jmb/filters/admin/options.py <==
# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import re
from copy import copy

import django
from django.utils import six
from django.contrib import admin
from django.contrib.admin.views.main import ChangeList

from jmb.filters import FilterSet

class ConfigurationError(Exception): pass


class AdvancedSearchModelAdmin(admin.ModelAdmin):
    """Model admin that will add a filterset built from advanced_search_fields
    Also allows to easily ovewrite widgets to implement autocompletion in advanced search
    just setting filterset_widgets dict as in::

      filterset_widgets = {
         'contacts': al.ChoiceWidget('contacts')
      }

    in this case al is autocompletion_ligh with a define autocompletion naed 'contacts'
    """
    filterset_widgets = {}

    def get_filterset_class(self, request):
        """Return a filterset_class suitable for advanced search
        first look if attribute ``advanced_search_fields``
        """
        fields = self.get_advanced_search_fields(request)
        if fields:
            class_name = 'Admin%sFilterset' % self.model.__name__
            meta = type(str('Meta'), (object,), {'fields': fields, 'model' : self.model})
            class_attrs = {
                'Meta' : meta,
                'fields' : fields,
                'FILTERSET_WIDGETS': self.filterset_widgets,
            }
            AdminFilterset = type(str(class_name), (FilterSet,), class_attrs)
            return AdminFilterset
        return None


    def get_advanced_search_fields(self, request):
        """
        Return the list of fields that should be used in advanced search

        :arg request: the request
        """
        return getattr(self, 'advanced_search_fields', None)

    # def lookup_allowed(self, lookup, value):
    #     ## we don't get here for fields in self._lookup_field
    #     ## since we prevent this pass within ModelAdmin.get_queryset
    #     ## when we pop all attributes used by out filterset
    #     ## so that these lines may be useless...
    #     if hasattr(self, '_lookup_names'):
    #         if lookup in self._lookup_names:
    #             return True

    #     return super(AdvancedSearchModelAdmin, self).lookup_allowed(lookup, value)

    def get_changelist(self, request):
        return AdvancedSearchChangeList

    def get_queryset(self, request):

        search_filterset_class = self.get_filterset_class(request)

        if search_filterset_class:
            self.search_filterset = search_filterset_class(request.GET)
            self.search_form = self.search_filterset.form
            self.search_form.is_valid()

            self._lookup_names = re.findall('name="(?P<name>[^\"]*)"', str(self.search_form))

            qs = self.search_filterset.qs
            # ordering is the only remainig action taken in ModelAdmin.queryset...
            ordering = self.get_ordering(request)
            if ordering:
                qs = qs.order_by(*ordering)
            return qs
        try:
            return admin.ModelAdmin.get_queryset(self, request)
        except:
            return admin.ModelAdmin.queryset(self, request)

    if django.VERSION[:2] < (1, 6):
        queryset = get_queryset
        del get_queryset

class AdvancedSearchChangeList(ChangeList):
    """
    A ChangeList that can hadle any filter condition
    added by the search_filterset's form.

    Django uses ModelAdmin's queryset method to initialize
    ChangeList's root_queryset::

       self.root_queryset = model_admin.get_queryset(request)

    and at the end of the :meth:`__init__` it handles all filtering
    due according to filters and the like::

       self.queryset = self.get_queryset(request)

   that uses it as base for the root_queryset::

       qs = self.root_queryset

    Beside handling all parameters contained in the filterset
    we use and pop ``_list_per_page`` attribute
    """
    def get_queryset(self, request):
        # save a copy of the GET parameters
        saved_params = copy(self.params)
        self.params.pop("_popup", None)
        self.params.pop("_json", None)
	# 1. query_str  2. list_per_page  3. settings.LIST_PER_PAGE
        self.list_per_page = int(self.params.pop(
            '_list_per_page', self.model_admin.get_list_per_page()
        ))
        ## ModelAdmin.get_queryset has already filtered the qs
        #  we now just need to hide the paramenters that belong to
        #  the filterset_class.form. Standard ChangeList would not
        #  know how to handle them
        if hasattr(self.model_admin, 'search_form'):
            search_form = self.model_admin.search_form

            if search_form.is_valid():
                for name in self.model_admin._lookup_names:
                    self.params.pop(name, None)
            else:
                msg = "search_form.is_valid return False (%r)" % search_form.errors
                raise ConfigurationError(msg)

        if django.VERSION[:2] >= (1, 6):
            qs_result = ChangeList.get_queryset(self, request)
        else:
            qs_result = ChangeList.get_query_set(self, request)

        # restore the self.params list so that templatetag 'result_list'
        # can compose it correctly. In particular ordering and filters
        # need to know all parameters

        self.params = saved_params

        return qs_result

    if django.VERSION[:2] < (1, 6):
        get_query_set = get_queryset

==> jmb.filters-0.4.3.tar.gz: jmb.filters-0.4.3/jmb/filters/admin/options.py <==
# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import re
from copy import copy

import django
from django.utils import six
from django.contrib import admin
from django.contrib.admin.views.main import ChangeList

from jmb.filters import FilterSet


class ConfigurationError(Exception):
    pass


class AdvancedSearchModelAdmin(admin.ModelAdmin):

    """Model admin that will add a filterset built from advanced_search_fields
    Also allows to easily ovewrite widgets to implement autocompletion in advanced search
    just setting filterset_widgets dict as in::

      filterset_widgets = {
         'contacts': al.ChoiceWidget('contacts')
      }

    in this case al is autocompletion_ligh with a define autocompletion naed 'contacts'
    """
    filterset_widgets = {}

    def get_filterset_class(self, request):
        """Return a filterset_class suitable for advanced search
        first look if attribute ``advanced_search_fields``
        """
        fields = self.get_advanced_search_fields(request)
        if fields:
            class_name = 'Admin%sFilterset' % self.model.__name__
            meta = type(str('Meta'), (object,), {'fields': fields, 'model': self.model})
            class_attrs = {
                'Meta': meta,
                'fields': fields,
                'FILTERSET_WIDGETS': self.filterset_widgets,
            }
            AdminFilterset = type(str(class_name), (FilterSet,), class_attrs)
            return AdminFilterset
        return None

    def get_advanced_search_fields(self, request):
        """
        Return the list of fields that should be used in advanced search

        :arg request: the request
        """
        return getattr(self, 'advanced_search_fields', None)

    # def lookup_allowed(self, lookup, value):
    # we don't get here for fields in self._lookup_field
    # since we prevent this pass within ModelAdmin.get_queryset
    # when we pop all attributes used by out filterset
    # so that these lines may be useless...
    #     if hasattr(self, '_lookup_names'):
    #         if lookup in self._lookup_names:
    #             return True

    #     return super(AdvancedSearchModelAdmin, self).lookup_allowed(lookup, value)

    def get_changelist(self, request):
        return AdvancedSearchChangeList

    def get_queryset(self, request):

        search_filterset_class = self.get_filterset_class(request)

        if search_filterset_class:
            self.search_filterset = search_filterset_class(request.GET)
            self.search_form = self.search_filterset.form
            self.search_form.is_valid()

            self._lookup_names = re.findall('name="(?P<name>[^\"]*)"', str(self.search_form))

            qs = self.search_filterset.qs
            # ordering is the only remainig action taken in ModelAdmin.queryset...
            ordering = self.get_ordering(request)
            if ordering:
                qs = qs.order_by(*ordering)
            return qs
        try:
            return admin.ModelAdmin.get_queryset(self, request)
        except:
            return admin.ModelAdmin.queryset(self, request)

    def get_list_per_page(self):
        """
        Set how many items appear on each paginated admin change list page.
        This value is conditioned by settings.LIST_PER_PAGE.
        Example:
            LIST_PER_PAGE = {
                'content_content': 50,
                'content': 10,
                'all': 100,
            }
        """
        ## when used with jmb.jadmin, jmb.jadmin offers JmbModelAdmin that implements
        ## a richer version of get_list_per_page
        return 50

    if django.VERSION[:2] < (1, 6):
        queryset = get_queryset
        del get_queryset


class AdvancedSearchChangeList(ChangeList):

    """
    A ChangeList that can hadle any filter condition
    added by the search_filterset's form.

    Django uses ModelAdmin's queryset method to initialize
    ChangeList's root_queryset::

       self.root_queryset = model_admin.get_queryset(request)

    and at the end of the :meth:`__init__` it handles all filtering
    due according to filters and the like::

       self.queryset = self.get_queryset(request)

   that uses it as base for the root_queryset::

       qs = self.root_queryset

    Beside handling all parameters contained in the filterset
    we use and pop ``_list_per_page`` attribute
    """

    def get_queryset(self, request):
        # save a copy of the GET parameters
        saved_params = copy(self.params)
        self.params.pop("_popup", None)
        self.params.pop("_json", None)
        # 1. query_str  2. list_per_page  3. settings.LIST_PER_PAGE
        self.list_per_page = int(self.params.pop(
            '_list_per_page', self.model_admin.get_list_per_page()
        ))
        # ModelAdmin.get_queryset has already filtered the qs
        #  we now just need to hide the paramenters that belong to
        #  the filterset_class.form. Standard ChangeList would not
        #  know how to handle them
        if hasattr(self.model_admin, 'search_form'):
            search_form = self.model_admin.search_form

            if search_form.is_valid():
                for name in self.model_admin._lookup_names:
                    self.params.pop(name, None)
            else:
                msg = "search_form.is_valid return False (%r)" % search_form.errors
                raise ConfigurationError(msg)

        if django.VERSION[:2] >= (1, 6):
            qs_result = ChangeList.get_queryset(self, request)
        else:
            qs_result = ChangeList.get_query_set(self, request)

        # restore the self.params list so that templatetag 'result_list'
        # can compose it correctly. In particular ordering and filters
        # need to know all parameters

        self.params = saved_params

        return qs_result

    if django.VERSION[:2] < (1, 6):
        get_query_set = get_queryset
